//  JULIA.C      Afficheur de fractals de Julia
//  Dos Mode 13 Pascal version (C) Pierric PERMEZEL   23 october 1993
//  GGI C version (C) Thomas MANGIN  17 november 1999

#include <ggi/ggi.h>
#include <stdio.h>

typedef struct
{
	double	MinX, MaxX, MinY, MaxY;	// Fractal Min & Max calcul Values
	double	StepX, StepY;		// Step of calcul
	double	Sqr_Diverge;		// Stop Calcul value
	int	MaxIteration;		// Nber max of Iteration
	double	ConstReal;		// Julia Constant Real part
	double	ConstImag;		//                Imaginary part
} fractal_t;

typedef struct
{
	int 	sizex;
	int 	sizey;
	int**	data;	
} buffer_t;

ggi_visual_t	Visual;

void openVisual(int sizex, int sizey)
{
	if (ggiInit() < 0)
	{
		fprintf(stderr,"Cant initialise lib-ggi\n");
		exit(1);
	}
	
	Visual = ggiOpen(NULL);

	if(Visual==NULL)
		ggiPanic("Couldn't open default visual!\n");

	if(ggiSetGraphMode(Visual, sizex, sizey, GGI_AUTO, GGI_AUTO, GT_16BIT))
		ggiPanic("Cannot set %dx%dx16 mode!\n",sizex,sizey);
}

void closeVisual()
{
	ggiClose(Visual);
	ggiExit();
}

void initializeBuffer (buffer_t* buffer, int sizex, int sizey)
{
	// No memory check
	int i;

	buffer->sizex = sizex;
	buffer->sizey = sizey;

	buffer->data = (int**)malloc(sizey*sizeof(int));
	for(i=0; i<sizey; i++)
		buffer->data[i]=(int*)malloc(sizex*sizeof(int));
}

void drawBuffer(buffer_t* buffer)
{
	int x,y;
	// No test to insure the size is the same
	for(y=0;y<buffer->sizey;y++)
		for(x=0;x<buffer->sizex;x++)
			ggiPutPixel(Visual,x,y,buffer->data[y][x]);
	ggiFlush(Visual);
}
/* Free Buffer */

void initializefractal (fractal_t* fractal, int sizex, int sizey)
{
	char	Answer;

	fractal->MinX = -2;
	fractal->MaxX = 2;
	fractal->MinY = -1.5;
	fractal->MaxY = 1.5;
	fractal->ConstReal = 0.3;
	fractal->ConstImag= 0.6;

	fractal->MaxIteration = 30;
	fractal->Sqr_Diverge = 8;

	printf ("\nMandelbrot Fractal Drawer.\n");
	printf ("(c) Octobre 1994   by Pierric PERMEZEL.\n");
	printf ("Last Modifications by Thomas  MANGIN\n\n");
	printf ("Equations : z0 = x + iy\n");
	printf ("            zn = z(n-1) + z0\n\n");
	printf ("X Min = %3.2f\n",fractal->MinX);
	printf ("X Max =  %3.2f\n",fractal->MaxX);
	printf ("Y Min = %3.2f\n",fractal->MinY);
	printf ("Y Max =  %3.2f\n",fractal->MaxY);
	printf ("Julia Constant Part    = %3.2f\n",fractal->ConstReal);
	printf ("Julia Imaginary Part   = %3.2f\n",fractal->ConstImag);
	printf ("Nber Max of Iterations = %4.0u\n",fractal->MaxIteration);
	printf ("Limit of divergence    = %4.0f\n\n",fractal->Sqr_Diverge);

	printf ("Do you want to change it (Y/N) : ");
	scanf ("%c",&Answer);

	if ((Answer=='y') || (Answer=='Y'))
	{
		printf("Min X = "); scanf("%f",&(fractal->MinX));
		printf("Max X = "); scanf("%f",&(fractal->MaxX));
		printf("Min Y = "); scanf("%f",&(fractal->MinY));
		printf("Max Y = "); scanf("%f",&(fractal->MaxY));
		printf("Nber max of iterations = "); scanf("%u",&(fractal->MaxIteration));
		printf("Limite of divergence   = "); scanf ("%f",&(fractal->Sqr_Diverge));
	}

	fractal->Sqr_Diverge = fractal->Sqr_Diverge*fractal->Sqr_Diverge;
	fractal->StepX = (fractal->MaxX-fractal->MinX)/sizex;
	fractal->StepY = (fractal->MaxY-fractal->MinY)/sizey;
}

void calculJulia (buffer_t* buffer,fractal_t* fractal)
{
	int	i,j;
	double	X,Y,Xc,Yc;
	double	Tmp;
	int	Iteration_Nber;

	for ( i=0; i < buffer->sizex; i++)
	{
		Xc = fractal->MinX + fractal->StepX*i;         	// Z0 = Xc + i*Yc
		for (j=0; j < buffer->sizey; j++)
		{
			Yc = fractal->MinY + fractal->StepY*(buffer->sizey-1-j);
			X = Xc;
			Y = Yc;

			Iteration_Nber = 0;
			do													// Zn = Sqr(Z(n-1)) + Z0
			{
				Tmp = X*X - Y*Y + fractal->ConstReal;
				Y = 2*X*Y + fractal->ConstImag;
				X = Tmp;
				Iteration_Nber++;
			} while ( (X*X+ Y*Y <= fractal->Sqr_Diverge) &&
				(Iteration_Nber < fractal->MaxIteration) ) ;

			if (Iteration_Nber == fractal->MaxIteration)
				buffer->data[j][i] = 0;
			else
				buffer->data[j][i] = (Iteration_Nber % fractal->MaxIteration)+1;
		}
	}
}

int main (void)
{
	int i;
	const int screenx = 640;
	const int screeny = 480;

	fractal_t fractal;
	buffer_t buffer;

	initializefractal (&fractal,screenx,screeny);
	initializeBuffer (&buffer,screenx, screeny);
	openVisual(screenx,screeny);
	calculJulia(&buffer,&fractal);
	drawBuffer(&buffer);
	getchar();
	getchar();
	closeVisual();
	return 0;
}
