//  MANDEL.C      Afficheur de fractals de Mandelbrot
//  Dos Mode 13 Pascal version (C) Pierric PERMEZEL   23 october 1993
//  C version (C) Thomas MANGIN  24 october 1993


// TAB VALUE IS 3

// INCLUDE
#include <stdio.h>
#include <dos.h>
#include <conio.h>

// DEFINE
#define	UINT	unsigned int

#define Draw_Mode   0x13        // BIOS MODE 800*600*256
#define ScreenX     320         // **** ONLY ET4000 ****
#define ScreenY     200
#define ColorNber   256
#define Text_Mode   0X03

// PROTOTYTPE
void Change_Video			(char);
void SetPixel	 			(UINT,UINT,char);
void Initialize_Values      (void);
void Draw_Mandel 			(void);


// GLOBAL VALUES
float   MinX, MaxX, MinY, MaxY;         // Fractal Min & Max calcul Values
float	StepX, StepY;					// Step of calcul
float	Sqr_Diverge;					// Stop Calcul value
UINT	MaxIteration;					// Nber max of Iteration

// MAIN PART
void main (void)
{
	Change_Video(Text_Mode);
	Initialize_Values();
	Change_Video(Draw_Mode);
	Draw_Mandel();
	getch();
	Change_Video(Text_Mode);
}

void Change_Video (char Video_Mode)
{
	asm {	push	ax
			xor	ah,ah
			mov	al,Video_Mode
			INT	10H
			pop	ax }
}

void SetPixel (UINT X, UINT Y, char Color)
{
    asm {   push  ax
            push  cx
            push  dx
            xor   ax,ax
            mov   ah,0X0C
            mov   al,Color
            mov   cx,X
            mov   dx,Y
            INT   10H
            pop   dx
			pop   cx
			pop   ax }
}

void Initialize_Values (void)
{
	char	Answer;

	MinX = -2.25;
	MaxX = 1.25;
	MinY = -1.33;
	MaxY = 1.33;
	MaxIteration = 255;			// A least the nber of colours
	Sqr_Diverge = 8;

	printf ("\nMandelbrot Fractal Drawer.\n");
	printf ("(c) Octobre 1994   by Pierric PERMEZEL.\n");
	printf ("Last Modifications by Thomas  MANGIN\n\n");
	printf ("Equations : z0 = x + iy\n");
	printf ("            zn = z(n-1) + z0\n\n");
	printf ("X Min = %3.2f\n",MinX);
	printf ("X Max =  %3.2f\n",MaxX);
	printf ("Y Min = %3.2f\n",MinY);
	printf ("Y Max =  %3.2f\n",MaxY);
	printf ("Nber Max of Iterations = %3.0u\n",MaxIteration);
	printf ("Limit of divergence    = %3.0f\n\n",Sqr_Diverge);

	printf ("Do you want to change it (Y/N) : ");
	scanf ("%c",&Answer);

	if ((Answer=='y') || (Answer=='Y'))
	{
		printf("Min X = "); scanf("%f",&MinX);
		printf("Max X = "); scanf("%f",&MaxX);
		printf("Min Y = "); scanf("%f",&MinY);
		printf("Max Y = "); scanf("%f",&MaxY);
		printf("Nber max of iterations = "); scanf("%u",&MaxIteration);
		printf("Limite of divergence   = "); scanf ("%f",&Sqr_Diverge);
	}

  Sqr_Diverge = Sqr_Diverge*Sqr_Diverge;
  StepX = (MaxX-MinX)/ScreenX;
  StepY = (MaxY-MinY)/ScreenY;
}

void Draw_Mandel (void)
{
	UINT	i,j;
	float	X,Y,Xc,Yc;
	float	Tmp;
	UINT	Iteration_Nber;

	for ( i=0; i < ScreenX; i++)
	{
		Xc = MinX + StepX*i;	  		             	// Z0 = Xc + i*Yc
		for (j=0; j < ScreenY; j++)
		{
			Yc = MinY + StepY*(ScreenY-1-j);
			X = Xc;
			Y = Yc;

			Iteration_Nber = 0;
            do                                          // Zn = Sqr(Z(n-1)) + Z0
			{
				Tmp = X*X - Y*Y + Xc;
				Y = 2*X*Y + Yc;
				X = Tmp;
				Iteration_Nber++;
			} while ( (X*X+ Y*Y <= Sqr_Diverge) &&
						 (Iteration_Nber < MaxIteration) ) ;

			if (Iteration_Nber == MaxIteration)
				SetPixel(i,j,0);
			else
				SetPixel(i,j,(Iteration_Nber%ColorNber)+1);
		}
	}
}
